package org.fouram.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang3.StringUtils;
import org.fouram.constants.WebConstants;
import org.fouram.core.base.controller.BaseController;
import org.fouram.core.plugin.weixin.cp.core.WXCpMailList;
import org.fouram.core.util.AppUtil;
import org.fouram.core.util.AppUtil.ResultConstant;
import org.fouram.core.util.ConfConfig;
import org.fouram.core.util.DateUtil;
import org.fouram.core.util.ExcelExportUtil;
import org.fouram.core.util.LoggerUtil;
import org.fouram.core.util.ResultUtil;
import org.fouram.dto.input.FindUsersDTO;
import org.fouram.dto.input.GetIngoreListDTO;
import org.fouram.dto.input.GetToBeConfirmedListDTO;
import org.fouram.dto.input.UpdateResultDTO;
import org.fouram.dto.output.GetIngoreListOutput;
import org.fouram.dto.output.GetToBeConfirmedListOutput;
import org.fouram.dto.output.SyncWeixinUserOutput;
import org.fouram.entity.SapOrg;
import org.fouram.entity.SapUser;
import org.fouram.enums.DelFlagEnum;
import org.fouram.service.SapCheckReportService;
import org.fouram.service.SapOrgService;
import org.fouram.service.SapUserService;
import org.fouram.service.SyncSapUserToWxService;
import org.fouram.service.WxDepartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import me.chanjar.weixin.cp.bean.WxCpOauth2UserInfo;

@Controller
@Api(value = "API接口")
@RequestMapping(value = "/API")
public class ApiController extends BaseController {

	@Autowired
	private SapOrgService sapOrgService;
	@Autowired
	private SapUserService sapUserService;
	@Autowired
	private WxDepartService wxDepartService;
	@Autowired
	private SapCheckReportService sapCheckReportService;
	@Autowired
	private SyncSapUserToWxService syncSapUserToWxService;

	private static final String TOKEN = "UserId";

	/**
	 * 获取授权url
	 *
	 * @return
	 */
	@PostMapping(value = "/getAuthUrl", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object getAuthUrl(String url) {
		try {
			String redirectUri = ConfConfig.getConfigString("wxCp.redirectUri") + "/API/wxCpLogin";
			String authUri = WXCpMailList.oauth2Service.buildAuthorizationUrl(redirectUri, url);
			System.out.println(authUri);
			return AppUtil.success(authUri, null);
		} catch (Exception e) {
			LoggerUtil.error(e);
			return AppUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	@PostMapping(value = "/wxCpLogin", produces = "application/json;charset=utf-8")
	@ResponseBody
	public void wxCpLogin(String code, String state, HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		WxCpOauth2UserInfo userInfo = WXCpMailList.oauth2Service.getUserInfo(code);
		String userId = ConfConfig.getConfigString("wxCp.mailList.userId");
		List<String> userIdList = Arrays.asList(userId.split(","));
		if (!userIdList.contains(userInfo.getUserId())) {
			response.getWriter().println("无权限访问！");
		} else {
			HttpSession httpSession = request.getSession();
			httpSession.setAttribute(TOKEN, userInfo.getUserId());
			response.sendRedirect(state);
		}
	}

	/**
	 * 根据部门id查询下级部门和用户
	 *
	 * @return
	 */
	@PostMapping(value = "/getChildDepartAndUserListByDepartId", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object getChildDepartAndUserListByDepartId(HttpServletRequest request) {
		if (!isLogin(request)) {
			return AppUtil.error(ResultConstant.NO_TOKEN, ResultConstant.TOKEN_ERR_MSG);
		}
		try {
			String departId = getPageData().getString("departId");
			SapOrg sapOrg = null;
			if (StringUtils.isBlank(departId) || WebConstants.TOP_DEPART.toString().equals(departId)) {
				departId = WebConstants.TOP_DEPART.toString();
				sapOrg = SapOrg.builder().id(WebConstants.TOP_DEPART.toString()).sapName(WebConstants.TOP_DEPART_NAME)
						.level("1").build();
			} else {
				sapOrg = sapOrgService.selectById(departId);
			}
			Map<String, Object> result = Maps.newHashMap();
			result.put("departInfo", sapOrg);
			result.put("departList", sapOrgService.selectListByParentId(departId));
			result.put("userList", sapUserService.selectPrimaryListByDepartIdAndLevel(departId));
			return AppUtil.success(result, null);
		} catch (Exception e) {
			LoggerUtil.error(e);
			return AppUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	/**
	 * 根据输入查询内容查询对应部门和用户
	 *
	 * @return
	 */
	@PostMapping(value = "/getDepartAndUserListByName", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object getDepartAndUserListByName(HttpServletRequest request) {
		if (!isLogin(request)) {
			return AppUtil.error(ResultConstant.NO_TOKEN, ResultConstant.TOKEN_ERR_MSG);
		}
		try {
			String name = getPageData().getString("name");
			Map<String, Object> result = Maps.newHashMap();
			result.put("departList", sapOrgService.selectListByName(name));
			result.put("userList", sapUserService.selectPrimaryListByName(name));
			return AppUtil.success(result, null);
		} catch (Exception e) {
			LoggerUtil.error(e);
			return AppUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	/**
	 * 根据id查询用户信息
	 *
	 * @return
	 */
	@PostMapping(value = "/getUserInfoByUserId", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object getUserInfoById(HttpServletRequest request) {
		if (!isLogin(request)) {
			return AppUtil.error(ResultConstant.NO_TOKEN, ResultConstant.TOKEN_ERR_MSG);
		}
		try {
			String userId = getPageData().getString("userId");
			return AppUtil.success(sapUserService.getDetailById(userId), null);
		} catch (Exception e) {
			LoggerUtil.error(e);
			return AppUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	/**
	 * 查询离职员工列表
	 *
	 * @return
	 */
	@PostMapping(value = "/getQuitUserList", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object getQuitUserList(HttpServletRequest request) {
		if (!isLogin(request)) {
			return AppUtil.error(ResultConstant.NO_TOKEN, ResultConstant.TOKEN_ERR_MSG);
		}
		try {
			return AppUtil.success(sapUserService.selectPrimaryListByDelFlag(DelFlagEnum.UNABLE.getCode()), null);
		} catch (Exception e) {
			LoggerUtil.error(e);
			return AppUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	@ApiResponses(@ApiResponse(response = GetToBeConfirmedListOutput.class, code = 200, message = "success"))
	@ApiOperation(value = "查询待确认员工列表")
	@PostMapping(value = "/getToBeConfirmedList", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object getToBeConfirmedList(HttpServletRequest request, @RequestBody GetToBeConfirmedListDTO dto) {
		if (!isLogin(request)) {
			return ResultUtil.error(ResultConstant.TOKEN_ERR_MSG);
		}
		try {
			if (WebConstants.CACHE.containsKey(WebConstants.CACHE_SYNC_SAP)) {
				return ResultUtil.error("数据正在同步，请稍等");
			}
			return ResultUtil.success(sapUserService.selectToBeConfirmedPrimaryList(dto), null);
		} catch (Exception e) {
			LoggerUtil.error(e);
			return ResultUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	@ApiOperation(value = "更新忽略")
	@PostMapping(value = "/updateIngore", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object updateIngore(HttpServletRequest request, @RequestBody UpdateResultDTO dto) {
		if (!isLogin(request)) {
			return ResultUtil.error(ResultConstant.TOKEN_ERR_MSG);
		}
		try {
			List<String> userIds = dto.getUserIds();
			if (StringUtils.isNotBlank(dto.getUserId())) {
				userIds = Lists.newArrayList(dto.getUserId());
			}
			sapUserService.updateIngore(userIds);
			return ResultUtil.success(null, "操作成功");
		} catch (Exception e) {
			LoggerUtil.error(e);
			return ResultUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	@ApiOperation(value = "更新同步")
	@PostMapping(value = "/updateSync", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object updateSync(HttpServletRequest request, @RequestBody UpdateResultDTO dto) {
		if (!isLogin(request)) {
			return ResultUtil.error(ResultConstant.TOKEN_ERR_MSG);
		}
		try {
			List<String> userIds = dto.getUserIds();
			if (StringUtils.isNotBlank(dto.getUserId())) {
				userIds = Lists.newArrayList(dto.getUserId());
			}
			sapUserService.updateSync(userIds);
			return ResultUtil.success(null, "操作成功");
		} catch (Exception e) {
			LoggerUtil.error(e);
			return ResultUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	@ApiOperation(value = "手动同步SAP用户数据")
	@ApiResponses(@ApiResponse(response = SyncWeixinUserOutput.class, code = 200, message = "success"))
	@PostMapping(value = "/syncWeixinUser", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object syncWeixinUser() {
		try {
			SyncWeixinUserOutput output = syncSapUserToWxService.syncWeixinUser();
			return ResultUtil.success(output, "操作成功");
		} catch (Exception e) {
			LoggerUtil.error(e);
			return ResultUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	@ApiResponses(@ApiResponse(response = GetIngoreListOutput.class, code = 200, message = "success"))
	@ApiOperation(value = "查询忽略列表")
	@PostMapping(value = "/getIngoreList", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object getCancelIngoreList(HttpServletRequest request, @RequestBody GetIngoreListDTO dto) {
		if (!isLogin(request)) {
			return AppUtil.error(ResultConstant.NO_TOKEN, ResultConstant.TOKEN_ERR_MSG);
		}
		try {
			return ResultUtil.success(sapUserService.selectIngorePrimaryList(dto), null);
		} catch (Exception e) {
			LoggerUtil.error(e);
			return AppUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	@ApiOperation(value = "取消忽略")
	@PostMapping(value = "/cancelIngore", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object cancelIngore(HttpServletRequest request, @RequestBody UpdateResultDTO dto) {
		if (!isLogin(request)) {
			return AppUtil.error(ResultConstant.NO_TOKEN, ResultConstant.TOKEN_ERR_MSG);
		}
		try {
			sapUserService.updateCancelIngore(dto.getUserId());
			return ResultUtil.success(null, "操作成功");
		} catch (Exception e) {
			LoggerUtil.error(e);
			return ResultUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	/**
	 * 删除离职员工
	 *
	 * @return
	 */
	@PostMapping(value = "/deleteQuitUserByUserId", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object deleteQuitUserByUserId(HttpServletRequest request) {
		if (!isLogin(request)) {
			return AppUtil.error(ResultConstant.NO_TOKEN, ResultConstant.TOKEN_ERR_MSG);
		}
		try {
			String userId = getPageData().getString("userId");
			if (StringUtils.isBlank(userId)) {
				return AppUtil.error("参数用户ID不可以为空");
			}
			SapUser sapUser = sapUserService.selectByUserId(userId);
			if (!DelFlagEnum.UNABLE.getCode().equals(sapUser.getDelFlag())) {
				return AppUtil.error("该用户ID不是离职数据");
			}
			// 操作用户ID
			String oprUserId = request.getSession().getAttribute(TOKEN).toString();
			sapUserService.deleteQuitUserByUserId(userId, oprUserId);
			return AppUtil.success();
		} catch (Exception e) {
			LoggerUtil.error(e);
			return AppUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	/**
	 * 判断是否经过授权
	 *
	 * @param request
	 * @return
	 */
	public boolean isLogin(HttpServletRequest request) {
//		HttpSession httpSession = request.getSession();
//		if (null == httpSession.getAttribute(TOKEN)) {
//			return false;
//		}
		return true;
	}

	@PostMapping(value = "/findDepartJSON", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object findDepartJSON(HttpServletRequest request) {
		try {
			return ResultUtil.success(wxDepartService.findDepartJSON(), "操作成功");
		} catch (Exception e) {
			LoggerUtil.error(e);
			return ResultUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	@PostMapping(value = "/findUserReports", produces = "application/json;charset=utf-8")
	@ResponseBody
	public Object findUsers(HttpServletRequest request, @RequestBody FindUsersDTO dto) {
		try {
			if (dto.getPageNumber() == null) {
				return ResultUtil.error("参数pageNumber为空");
			}
			Object result = sapCheckReportService.findUserReportPage(dto.getStartDate(), dto.getEndDate(),
					dto.getDepartId(), dto.getName(), dto.getStatus(), dto.getPageSize(), dto.getPageNumber());
			return ResultUtil.success(result, "操作成功");
		} catch (Exception e) {
			LoggerUtil.error(e);
			return ResultUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}

	@PostMapping(value = "/exportUserReportExcel")
	@ResponseBody
	public Object exportUserReportExcel(HttpServletResponse response, @RequestBody FindUsersDTO dto) {
		try {
			if (StringUtils.isBlank(dto.getStartDate())) {
				return ResultUtil.error("开始日期不可以为空");
			}
			if (StringUtils.isBlank(dto.getEndDate())) {
				return ResultUtil.error("结束日期不可以为空");
			}
			String ysFilePath = "/public/excel/userReport.xlsx";
			// 汇总
			String hzFilePath = "/public/excel/userReport" + DateUtil.getTimeStamp() + ".xlsx";
			List<Map<String, Object>> hzList = sapCheckReportService.selectReportDayList(dto.getStartDate(),
					dto.getEndDate(), dto.getDepartId(), dto.getName());
			ExcelExportUtil.exportExcelFile(getRootPath() + ysFilePath, hzList, getRootPath() + hzFilePath, 0, 0);
			ysFilePath = hzFilePath;
			// 明细
			String resultFilePath = "/public/excel/userReport" + DateUtil.getTimeStamp() + ".xlsx";
			List<Map<String, Object>> mxList = sapCheckReportService.findUserReports(dto.getStartDate(),
					dto.getEndDate(), dto.getDepartId(), dto.getName(), dto.getStatus(), null, null);
			ExcelExportUtil.exportExcelFile(getRootPath() + ysFilePath, mxList, getRootPath() + resultFilePath, 1, 1);
			return ResultUtil.success(resultFilePath, "操作成功");
		} catch (Exception e) {
			LoggerUtil.error(e);
			return ResultUtil.error(ResultConstant.WEB_ERR_MSG);
		}
	}
}
