package org.fouram.service;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.fouram.constants.WebConstants;
import org.fouram.core.base.service.BaseService;
import org.fouram.core.util.DateUtil;
import org.fouram.core.util.LoggerUtil;
import org.fouram.dto.output.SyncWeixinUserOutput;
import org.fouram.enums.DelFlagEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class SyncSapUserToWxService extends BaseService {

	@Autowired
	private WxUserService wxUserService;
	@Autowired
	private SapUserService sapUserService;
	@Autowired
	private WxDepartService wxDepartService;

	public SyncWeixinUserOutput syncWeixinUser() throws Exception {
		if (WebConstants.CACHE.containsKey(WebConstants.CACHE_SYNC_SAP)) {
			return SyncWeixinUserOutput.builder().state("同步中").time(WebConstants.CACHE.get(WebConstants.CACHE_SYNC_SAP))
					.build();
		}
		ExecutorService threadPool = Executors.newSingleThreadExecutor();
		threadPool.execute(new Runnable() {
			@Override
			public void run() {
				LoggerUtil.info("同步SAP执行开始:" + DateUtil.getTime());
				taskBiz();
				LoggerUtil.info("同步SAP执行开始:" + DateUtil.getTime());
			}
		});
		threadPool.shutdown();// 任务执行完毕，关闭线程池
		return SyncWeixinUserOutput.builder().state("正在同步，请稍等").build();
	}

	@Transactional
	public void taskBiz() {
		try {
			WebConstants.CACHE.put(WebConstants.CACHE_SYNC_SAP, DateUtil.getTime());
			// 更新微信部门
			wxDepartService.saveWxDeparts();
			LoggerUtil.info("saveWxDeparts:" + DateUtil.getTime());
			// 更新微信用户
			wxUserService.saveWxUsers();
			LoggerUtil.info("saveWxUsers:" + DateUtil.getTime());
			// 更新所有数据无效
			sapUserService.updateAllDelFlag(DelFlagEnum.UNABLE.getCode());
			LoggerUtil.info("updateAllDelFlag:" + DateUtil.getTime());
			// 保存SAP用户数据
			sapUserService.saveOrUpdateSapUsers();
			LoggerUtil.info("saveOrUpdateSapUsers:" + DateUtil.getTime());
			// 更新异常情况，敏感数据、入职等
			sapUserService.updateAbnormalResult();
			LoggerUtil.info("updateAbnormalResult:" + DateUtil.getTime());
			// 更新用户头像
			sapUserService.updateAvatars();
			LoggerUtil.info("updateAvatars:" + DateUtil.getTime());
			// NORMAL数据更新同步企业微信
			sapUserService.updateAllNormalSyncWxResult();
			LoggerUtil.info("updateAllNormalSyncWxResult:" + DateUtil.getTime());
		} catch (Exception e) {
			LoggerUtil.error(e);
		} finally {
			WebConstants.CACHE.remove(WebConstants.CACHE_SYNC_SAP);
		}
	}
}
